% Created by Billy Hawkins on October 3rd, 2011
% 
% INPUTS:
% depData       They data from DFW consistion of spot ready times,
%               unimpeded times, runway ready times, and aircraft types in that order.
% separationT   4x4 matrix of separation times between aircraft types
%
% OUTPUTS:
% FCFSt         The FCFS takeoff times for each aircraft
% alpha         The runway ready times for each aircraft
% delta         Matrix of separation times between every possible pair of
%               aircraft
%
% This function assigns takeoff times for a FCFS order while obeying
% separation requirements.

function [FCFSt, alpha, delta] = newRunwayTimes(depData, separationT)

    % Get vectors of the alphas and the aircraft types
    alpha = depData(:,3);
    aType = depData(:,4);

    % Create matrix of required separations between all pairs of adjacent
    % flights i, j
    delta = zeros(size(depData,1));
    for i = 1 : size(depData,1)
        for j = 1 : size(depData,1)
            if (i ~= j)
                delta(i,j) = separationT(aType(i), aType(j));
            end
        end
    end
    
    % Vector of new alpha runway ready times
    FCFSt = zeros(size(depData,1),1);
    
    % Determine FCFS takeoff times based on original alphas and obeying
    % separation times
    for i = 1 : size(depData,1)
        if (i == 1 || (i == 2 && alpha(i) - FCFSt(i-1) >= delta(i-1, i)))
            FCFSt(i) = alpha(i);
        elseif (i == 2 && alpha(i) - FCFSt(i-1) < delta(i-1,i))
            FCFSt(i) = FCFSt(i-1) + delta(i-1, i);
        else
            FCFSt(i) = max(alpha(i), max(FCFSt(i-1) + delta(i-1, i), FCFSt(i-2) + delta(i-2,i)));
        end
    end
end